@@ -36,7 +36,7 @@
 #include <stdlib.h>
 #include <math.h>
 
-#include <glad/glad.h>
+#include <glad/gl.h>
 #include <GLFW/glfw3.h>
 
 #include <linmath.h>
@@ -165,7 +165,7 @@ void CrossProduct( vertex_t a, vertex_t b, vertex_t c, vertex_t *n )
    v2 = c.y - a.y;
    v3 = c.z - a.z;
 
-   n->x = u2 * v3 - v2 * v3;
+   n->x = u2 * v3 - v2 * u3;
    n->y = u3 * v1 - v3 * u1;
    n->z = u1 * v2 - v1 * u2;
 }
@@ -302,7 +302,7 @@ void cursor_position_callback( GLFWwindow* window, double x, double y )
  * The Boing ball is sphere in which each facet is a rectangle.
  * Facet colors alternate between red and white.
  * The ball is built by stacking latitudinal circles.  Each circle is composed
- * of a widely-separated set of points, so that each facet is noticably large.
+ * of a widely-separated set of points, so that each facet is noticeably large.
  *****************************************************************************/
 void DrawBoingBall( void )
 {
@@ -446,7 +446,7 @@ void DrawBoingBallBand( GLfloat long_lo,
    static int colorToggle = 0;
 
   /*
-   * Iterate thru the points of a latitude circle.
+   * Iterate through the points of a latitude circle.
    * A latitude circle is a 2D set of X,Z points.
    */
    for ( lat_deg = 0;
@@ -642,7 +642,7 @@ int main( void )
    glfwSetCursorPosCallback(window, cursor_position_callback);
 
    glfwMakeContextCurrent(window);
-   gladLoadGLLoader((GLADloadproc) glfwGetProcAddress);
+   gladLoadGL(glfwGetProcAddress);
    glfwSwapInterval( 1 );
 
    glfwGetFramebufferSize(window, &width, &height);
