@@ -1,6 +1,6 @@
 //========================================================================
 // Clipboard test program
-// Copyright (c) Camilla Berglund <elmindreda@glfw.org>
+// Copyright (c) Camilla Löwy <elmindreda@glfw.org>
 //
 // This software is provided 'as-is', without any express or implied
 // warranty. In no event will the authors be held liable for any damages
@@ -27,7 +27,7 @@
 //
 //========================================================================
 
-#include <glad/glad.h>
+#include <glad/gl.h>
 #include <GLFW/glfw3.h>
 
 #include <stdio.h>
@@ -67,7 +67,7 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
             {
                 const char* string;
 
-                string = glfwGetClipboardString(window);
+                string = glfwGetClipboardString(NULL);
                 if (string)
                     printf("Clipboard contains \"%s\"\n", string);
                 else
@@ -79,18 +79,13 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
             if (mods == MODIFIER)
             {
                 const char* string = "Hello GLFW World!";
-                glfwSetClipboardString(window, string);
+                glfwSetClipboardString(NULL, string);
                 printf("Setting clipboard to \"%s\"\n", string);
             }
             break;
     }
 }
 
-static void framebuffer_size_callback(GLFWwindow* window, int width, int height)
-{
-    glViewport(0, 0, width, height);
-}
-
 int main(int argc, char** argv)
 {
     int ch;
@@ -128,15 +123,10 @@ int main(int argc, char** argv)
     }
 
     glfwMakeContextCurrent(window);
-    gladLoadGLLoader((GLADloadproc) glfwGetProcAddress);
+    gladLoadGL(glfwGetProcAddress);
     glfwSwapInterval(1);
 
     glfwSetKeyCallback(window, key_callback);
-    glfwSetFramebufferSizeCallback(window, framebuffer_size_callback);
-
-    glMatrixMode(GL_PROJECTION);
-    glOrtho(-1.f, 1.f, -1.f, 1.f, -1.f, 1.f);
-    glMatrixMode(GL_MODELVIEW);
 
     glClearColor(0.5f, 0.5f, 0.5f, 0);
 
@@ -144,9 +134,6 @@ int main(int argc, char** argv)
     {
         glClear(GL_COLOR_BUFFER_BIT);
 
-        glColor3f(0.8f, 0.2f, 0.4f);
-        glRectf(-0.5f, -0.5f, 0.5f, 0.5f);
-
         glfwSwapBuffers(window);
         glfwWaitEvents();
     }
