@@ -1,6 +1,6 @@
 //========================================================================
 // Cursor & input mode tests
-// Copyright (c) Camilla Berglund <elmindreda@glfw.org>
+// Copyright (c) Camilla Löwy <elmindreda@glfw.org>
 //
 // This software is provided 'as-is', without any express or implied
 // warranty. In no event will the authors be held liable for any damages
@@ -30,7 +30,7 @@
 //
 //========================================================================
 
-#include <glad/glad.h>
+#include <glad/gl.h>
 #include <GLFW/glfw3.h>
 
 #if defined(_MSC_VER)
@@ -42,8 +42,26 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "linmath.h"
+
 #define CURSOR_FRAME_COUNT 60
 
+static const char* vertex_shader_text =
+"#version 110\n"
+"uniform mat4 MVP;\n"
+"attribute vec2 vPos;\n"
+"void main()\n"
+"{\n"
+"    gl_Position = MVP * vec4(vPos, 0.0, 1.0);\n"
+"}\n";
+
+static const char* fragment_shader_text =
+"#version 110\n"
+"void main()\n"
+"{\n"
+"    gl_FragColor = vec4(1.0);\n"
+"}\n";
+
 static double cursor_x;
 static double cursor_y;
 static int swap_interval = 1;
@@ -51,6 +69,7 @@ static int wait_events = GLFW_TRUE;
 static int animate_cursor = GLFW_FALSE;
 static int track_cursor = GLFW_FALSE;
 static GLFWcursor* standard_cursors[6];
+static GLFWcursor* tracking_cursor = NULL;
 
 static void error_callback(int error, const char* description)
 {
@@ -93,6 +112,36 @@ static GLFWcursor* create_cursor_frame(float t)
     return glfwCreateCursor(&image, image.width / 2, image.height / 2);
 }
 
+static GLFWcursor* create_tracking_cursor(void)
+{
+    int i = 0, x, y;
+    unsigned char buffer[32 * 32 * 4];
+    const GLFWimage image = { 32, 32, buffer };
+
+    for (y = 0;  y < image.width;  y++)
+    {
+        for (x = 0;  x < image.height;  x++)
+        {
+            if (x == 7 || y == 7)
+            {
+                buffer[i++] = 255;
+                buffer[i++] = 0;
+                buffer[i++] = 0;
+                buffer[i++] = 255;
+            }
+            else
+            {
+                buffer[i++] = 0;
+                buffer[i++] = 0;
+                buffer[i++] = 0;
+                buffer[i++] = 0;
+            }
+        }
+    }
+
+    return glfwCreateCursor(&image, 7, 7);
+}
+
 static void cursor_position_callback(GLFWwindow* window, double x, double y)
 {
     printf("%0.3f: Cursor position: %f %f (%+f %+f)\n",
@@ -132,6 +181,7 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
 
         case GLFW_KEY_N:
             glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_NORMAL);
+            glfwGetCursorPos(window, &cursor_x, &cursor_y);
             printf("(( cursor is normal ))\n");
             break;
 
@@ -145,6 +195,22 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
             printf("(( cursor is hidden ))\n");
             break;
 
+        case GLFW_KEY_R:
+            if (!glfwRawMouseMotionSupported())
+                break;
+
+            if (glfwGetInputMode(window, GLFW_RAW_MOUSE_MOTION))
+            {
+                glfwSetInputMode(window, GLFW_RAW_MOUSE_MOTION, GLFW_FALSE);
+                printf("(( raw input is disabled ))\n");
+            }
+            else
+            {
+                glfwSetInputMode(window, GLFW_RAW_MOUSE_MOTION, GLFW_TRUE);
+                printf("(( raw input is enabled ))\n");
+            }
+            break;
+
         case GLFW_KEY_SPACE:
             swap_interval = 1 - swap_interval;
             printf("(( swap interval: %i ))\n", swap_interval);
@@ -158,8 +224,47 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
 
         case GLFW_KEY_T:
             track_cursor = !track_cursor;
+            if (track_cursor)
+                glfwSetCursor(window, tracking_cursor);
+            else
+                glfwSetCursor(window, NULL);
+
             break;
 
+        case GLFW_KEY_P:
+        {
+            double x, y;
+            glfwGetCursorPos(window, &x, &y);
+
+            printf("Query before set: %f %f (%+f %+f)\n",
+                   x, y, x - cursor_x, y - cursor_y);
+            cursor_x = x;
+            cursor_y = y;
+
+            glfwSetCursorPos(window, cursor_x, cursor_y);
+            glfwGetCursorPos(window, &x, &y);
+
+            printf("Query after set: %f %f (%+f %+f)\n",
+                   x, y, x - cursor_x, y - cursor_y);
+            cursor_x = x;
+            cursor_y = y;
+            break;
+        }
+
+        case GLFW_KEY_UP:
+            glfwSetCursorPos(window, 0, 0);
+            glfwGetCursorPos(window, &cursor_x, &cursor_y);
+            break;
+
+        case GLFW_KEY_DOWN:
+        {
+            int width, height;
+            glfwGetWindowSize(window, &width, &height);
+            glfwSetCursorPos(window, width - 1, height - 1);
+            glfwGetCursorPos(window, &cursor_x, &cursor_y);
+            break;
+        }
+
         case GLFW_KEY_0:
             glfwSetCursor(window, NULL);
             break;
@@ -187,6 +292,31 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
         case GLFW_KEY_6:
             glfwSetCursor(window, standard_cursors[5]);
             break;
+
+        case GLFW_KEY_F11:
+        case GLFW_KEY_ENTER:
+        {
+            static int x, y, width, height;
+
+            if (mods != GLFW_MOD_ALT)
+                return;
+
+            if (glfwGetWindowMonitor(window))
+                glfwSetWindowMonitor(window, NULL, x, y, width, height, 0);
+            else
+            {
+                GLFWmonitor* monitor = glfwGetPrimaryMonitor();
+                const GLFWvidmode* mode = glfwGetVideoMode(monitor);
+                glfwGetWindowPos(window, &x, &y);
+                glfwGetWindowSize(window, &width, &height);
+                glfwSetWindowMonitor(window, monitor,
+                                     0, 0, mode->width, mode->height,
+                                     mode->refreshRate);
+            }
+
+            glfwGetCursorPos(window, &cursor_x, &cursor_y);
+            break;
+        }
     }
 }
 
@@ -196,12 +326,21 @@ int main(void)
     GLFWwindow* window;
     GLFWcursor* star_cursors[CURSOR_FRAME_COUNT];
     GLFWcursor* current_frame = NULL;
+    GLuint vertex_buffer, vertex_shader, fragment_shader, program;
+    GLint mvp_location, vpos_location;
 
     glfwSetErrorCallback(error_callback);
 
     if (!glfwInit())
         exit(EXIT_FAILURE);
 
+    tracking_cursor = create_tracking_cursor();
+    if (!tracking_cursor)
+    {
+        glfwTerminate();
+        exit(EXIT_FAILURE);
+    }
+
     for (i = 0;  i < CURSOR_FRAME_COUNT;  i++)
     {
         star_cursors[i] = create_cursor_frame(i / (float) CURSOR_FRAME_COUNT);
@@ -231,6 +370,9 @@ int main(void)
         }
     }
 
+    glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 2);
+    glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 0);
+
     window = glfwCreateWindow(640, 480, "Cursor Test", NULL, NULL);
     if (!window)
     {
@@ -239,7 +381,31 @@ int main(void)
     }
 
     glfwMakeContextCurrent(window);
-    gladLoadGLLoader((GLADloadproc) glfwGetProcAddress);
+    gladLoadGL(glfwGetProcAddress);
+
+    glGenBuffers(1, &vertex_buffer);
+    glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer);
+
+    vertex_shader = glCreateShader(GL_VERTEX_SHADER);
+    glShaderSource(vertex_shader, 1, &vertex_shader_text, NULL);
+    glCompileShader(vertex_shader);
+
+    fragment_shader = glCreateShader(GL_FRAGMENT_SHADER);
+    glShaderSource(fragment_shader, 1, &fragment_shader_text, NULL);
+    glCompileShader(fragment_shader);
+
+    program = glCreateProgram();
+    glAttachShader(program, vertex_shader);
+    glAttachShader(program, fragment_shader);
+    glLinkProgram(program);
+
+    mvp_location = glGetUniformLocation(program, "MVP");
+    vpos_location = glGetAttribLocation(program, "vPos");
+
+    glEnableVertexAttribArray(vpos_location);
+    glVertexAttribPointer(vpos_location, 2, GL_FLOAT, GL_FALSE,
+                          sizeof(vec2), (void*) 0);
+    glUseProgram(program);
 
     glfwGetCursorPos(window, &cursor_x, &cursor_y);
     printf("Cursor position: %f %f\n", cursor_x, cursor_y);
@@ -255,24 +421,33 @@ int main(void)
         {
             int wnd_width, wnd_height, fb_width, fb_height;
             float scale;
+            vec2 vertices[4];
+            mat4x4 mvp;
 
             glfwGetWindowSize(window, &wnd_width, &wnd_height);
             glfwGetFramebufferSize(window, &fb_width, &fb_height);
 
-            scale = (float) fb_width / (float) wnd_width;
-
             glViewport(0, 0, fb_width, fb_height);
 
-            glMatrixMode(GL_PROJECTION);
-            glLoadIdentity();
-            glOrtho(0.f, fb_width, 0.f, fb_height, 0.f, 1.f);
+            scale = (float) fb_width / (float) wnd_width;
+            vertices[0][0] = 0.5f;
+            vertices[0][1] = (float) (fb_height - floor(cursor_y * scale) - 1.f + 0.5f);
+            vertices[1][0] = (float) fb_width + 0.5f;
+            vertices[1][1] = (float) (fb_height - floor(cursor_y * scale) - 1.f + 0.5f);
+            vertices[2][0] = (float) floor(cursor_x * scale) + 0.5f;
+            vertices[2][1] = 0.5f;
+            vertices[3][0] = (float) floor(cursor_x * scale) + 0.5f;
+            vertices[3][1] = (float) fb_height + 0.5f;
 
-            glBegin(GL_LINES);
-            glVertex2f(0.f, (GLfloat) (fb_height - cursor_y * scale));
-            glVertex2f((GLfloat) fb_width, (GLfloat) (fb_height - cursor_y * scale));
-            glVertex2f((GLfloat) cursor_x * scale, 0.f);
-            glVertex2f((GLfloat) cursor_x * scale, (GLfloat) fb_height);
-            glEnd();
+            glBufferData(GL_ARRAY_BUFFER,
+                         sizeof(vertices),
+                         vertices,
+                         GL_STREAM_DRAW);
+
+            mat4x4_ortho(mvp, 0.f, (float) fb_width, 0.f, (float) fb_height, 0.f, 1.f);
+            glUniformMatrix4fv(mvp_location, 1, GL_FALSE, (const GLfloat*) mvp);
+
+            glDrawArrays(GL_LINES, 0, 4);
         }
 
         glfwSwapBuffers(window);
