@@ -1,6 +1,6 @@
 //========================================================================
 // Iconify/restore test program
-// Copyright (c) Camilla Berglund <elmindreda@glfw.org>
+// Copyright (c) Camilla Löwy <elmindreda@glfw.org>
 //
 // This software is provided 'as-is', without any express or implied
 // warranty. In no event will the authors be held liable for any damages
@@ -28,7 +28,7 @@
 //
 //========================================================================
 
-#include <glad/glad.h>
+#include <glad/gl.h>
 #include <GLFW/glfw3.h>
 
 #include <stdio.h>
@@ -45,7 +45,6 @@ static void usage(void)
     printf("  -a create windows for all monitors\n");
     printf("  -f create full screen window(s)\n");
     printf("  -h show this help\n");
-    printf("  -n no automatic iconification of full screen windows\n");
 }
 
 static void error_callback(int error, const char* description)
@@ -76,6 +75,18 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
         case GLFW_KEY_ESCAPE:
             glfwSetWindowShouldClose(window, GLFW_TRUE);
             break;
+        case GLFW_KEY_A:
+            glfwSetWindowAttrib(window, GLFW_AUTO_ICONIFY, !glfwGetWindowAttrib(window, GLFW_AUTO_ICONIFY));
+            break;
+        case GLFW_KEY_B:
+            glfwSetWindowAttrib(window, GLFW_RESIZABLE, !glfwGetWindowAttrib(window, GLFW_RESIZABLE));
+            break;
+        case GLFW_KEY_D:
+            glfwSetWindowAttrib(window, GLFW_DECORATED, !glfwGetWindowAttrib(window, GLFW_DECORATED));
+            break;
+        case GLFW_KEY_F:
+            glfwSetWindowAttrib(window, GLFW_FLOATING, !glfwGetWindowAttrib(window, GLFW_FLOATING));
+            break;
         case GLFW_KEY_F11:
         case GLFW_KEY_ENTER:
         {
@@ -116,8 +127,6 @@ static void window_size_callback(GLFWwindow* window, int width, int height)
 static void framebuffer_size_callback(GLFWwindow* window, int width, int height)
 {
     printf("%0.2f Framebuffer resized to %ix%i\n", glfwGetTime(), width, height);
-
-    glViewport(0, 0, width, height);
 }
 
 static void window_focus_callback(GLFWwindow* window, int focused)
@@ -131,29 +140,23 @@ static void window_iconify_callback(GLFWwindow* window, int iconified)
 {
     printf("%0.2f Window %s\n",
            glfwGetTime(),
-           iconified ? "iconified" : "restored");
+           iconified ? "iconified" : "uniconified");
+}
+
+static void window_maximize_callback(GLFWwindow* window, int maximized)
+{
+    printf("%0.2f Window %s\n",
+           glfwGetTime(),
+           maximized ? "maximized" : "unmaximized");
 }
 
 static void window_refresh_callback(GLFWwindow* window)
 {
-    int width, height;
-
     printf("%0.2f Window refresh\n", glfwGetTime());
 
-    glfwGetFramebufferSize(window, &width, &height);
-
     glfwMakeContextCurrent(window);
 
-    glEnable(GL_SCISSOR_TEST);
-
-    glScissor(0, 0, width, height);
-    glClearColor(0, 0, 0, 0);
     glClear(GL_COLOR_BUFFER_BIT);
-
-    glScissor(0, 0, 640, 480);
-    glClearColor(1, 1, 1, 0);
-    glClear(GL_COLOR_BUFFER_BIT);
-
     glfwSwapBuffers(window);
 }
 
@@ -188,7 +191,7 @@ static GLFWwindow* create_window(GLFWmonitor* monitor)
     }
 
     glfwMakeContextCurrent(window);
-    gladLoadGLLoader((GLADloadproc) glfwGetProcAddress);
+    gladLoadGL(glfwGetProcAddress);
 
     return window;
 }
@@ -196,7 +199,7 @@ static GLFWwindow* create_window(GLFWmonitor* monitor)
 int main(int argc, char** argv)
 {
     int ch, i, window_count;
-    int auto_iconify = GLFW_TRUE, fullscreen = GLFW_FALSE, all_monitors = GLFW_FALSE;
+    int fullscreen = GLFW_FALSE, all_monitors = GLFW_FALSE;
     GLFWwindow** windows;
 
     while ((ch = getopt(argc, argv, "afhn")) != -1)
@@ -215,10 +218,6 @@ int main(int argc, char** argv)
                 fullscreen = GLFW_TRUE;
                 break;
 
-            case 'n':
-                auto_iconify = GLFW_FALSE;
-                break;
-
             default:
                 usage();
                 exit(EXIT_FAILURE);
@@ -230,8 +229,6 @@ int main(int argc, char** argv)
     if (!glfwInit())
         exit(EXIT_FAILURE);
 
-    glfwWindowHint(GLFW_AUTO_ICONIFY, auto_iconify);
-
     if (fullscreen && all_monitors)
     {
         int monitor_count;
@@ -266,6 +263,7 @@ int main(int argc, char** argv)
         glfwSetWindowSizeCallback(windows[i], window_size_callback);
         glfwSetWindowFocusCallback(windows[i], window_focus_callback);
         glfwSetWindowIconifyCallback(windows[i], window_iconify_callback);
+        glfwSetWindowMaximizeCallback(windows[i], window_maximize_callback);
         glfwSetWindowRefreshCallback(windows[i], window_refresh_callback);
 
         window_refresh_callback(windows[i]);
