@@ -1,6 +1,6 @@
 //========================================================================
 // Monitor information tool
-// Copyright (c) Camilla Berglund <elmindreda@glfw.org>
+// Copyright (c) Camilla Löwy <elmindreda@glfw.org>
 //
 // This software is provided 'as-is', without any express or implied
 // warranty. In no event will the authors be held liable for any damages
@@ -28,7 +28,7 @@
 //
 //========================================================================
 
-#include <glad/glad.h>
+#include <glad/gl.h>
 #include <GLFW/glfw3.h>
 
 #include <stdio.h>
@@ -49,16 +49,24 @@ static void usage(void)
     printf("       monitors -h\n");
 }
 
+static int euclid(int a, int b)
+{
+    return b ? euclid(b, a % b) : a;
+}
+
 static const char* format_mode(const GLFWvidmode* mode)
 {
     static char buffer[512];
+    const int gcd = euclid(mode->width, mode->height);
 
-    sprintf(buffer,
-            "%i x %i x %i (%i %i %i) %i Hz",
-            mode->width, mode->height,
-            mode->redBits + mode->greenBits + mode->blueBits,
-            mode->redBits, mode->greenBits, mode->blueBits,
-            mode->refreshRate);
+    snprintf(buffer,
+             sizeof(buffer),
+             "%i x %i x %i (%i:%i) (%i %i %i) %i Hz",
+             mode->width, mode->height,
+             mode->redBits + mode->greenBits + mode->blueBits,
+             mode->width / gcd, mode->height / gcd,
+             mode->redBits, mode->greenBits, mode->blueBits,
+             mode->refreshRate);
 
     buffer[sizeof(buffer) - 1] = '\0';
     return buffer;
@@ -84,21 +92,29 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
 
 static void list_modes(GLFWmonitor* monitor)
 {
-    int count, x, y, widthMM, heightMM, i;
+    int count, x, y, width_mm, height_mm, i;
+    int workarea_x, workarea_y, workarea_width, workarea_height;
+    float xscale, yscale;
+
     const GLFWvidmode* mode = glfwGetVideoMode(monitor);
     const GLFWvidmode* modes = glfwGetVideoModes(monitor, &count);
 
     glfwGetMonitorPos(monitor, &x, &y);
-    glfwGetMonitorPhysicalSize(monitor, &widthMM, &heightMM);
+    glfwGetMonitorPhysicalSize(monitor, &width_mm, &height_mm);
+    glfwGetMonitorContentScale(monitor, &xscale, &yscale);
+    glfwGetMonitorWorkarea(monitor, &workarea_x, &workarea_y, &workarea_width, &workarea_height);
 
     printf("Name: %s (%s)\n",
            glfwGetMonitorName(monitor),
            glfwGetPrimaryMonitor() == monitor ? "primary" : "secondary");
     printf("Current mode: %s\n", format_mode(mode));
     printf("Virtual position: %i %i\n", x, y);
+    printf("Content scale: %f %f\n", xscale, yscale);
 
     printf("Physical size: %i x %i mm (%0.2f dpi)\n",
-           widthMM, heightMM, mode->width * 25.4f / widthMM);
+           width_mm, height_mm, mode->width * 25.4f / width_mm);
+    printf("Monitor work area: pos=(%d,%d)  size=(%dx%d)\n",
+            workarea_x, workarea_y, workarea_width, workarea_height);
 
     printf("Modes:\n");
 
@@ -150,7 +166,7 @@ static void test_modes(GLFWmonitor* monitor)
         glfwSetKeyCallback(window, key_callback);
 
         glfwMakeContextCurrent(window);
-        gladLoadGLLoader((GLADloadproc) glfwGetProcAddress);
+        gladLoadGL(glfwGetProcAddress);
         glfwSwapInterval(1);
 
         glfwSetTime(0.0);
