@@ -1,6 +1,6 @@
 //========================================================================
 // Vsync enabling test
-// Copyright (c) Camilla Berglund <elmindreda@glfw.org>
+// Copyright (c) Camilla Löwy <elmindreda@glfw.org>
 //
 // This software is provided 'as-is', without any express or implied
 // warranty. In no event will the authors be held liable for any damages
@@ -28,14 +28,41 @@
 //
 //========================================================================
 
-#include <glad/glad.h>
+#include <glad/gl.h>
 #include <GLFW/glfw3.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 
-#include "getopt.h"
+#include "linmath.h"
+
+static const struct
+{
+    float x, y;
+} vertices[4] =
+{
+    { -0.25f, -1.f },
+    {  0.25f, -1.f },
+    {  0.25f,  1.f },
+    { -0.25f,  1.f }
+};
+
+static const char* vertex_shader_text =
+"#version 110\n"
+"uniform mat4 MVP;\n"
+"attribute vec2 vPos;\n"
+"void main()\n"
+"{\n"
+"    gl_Position = MVP * vec4(vPos, 0.0, 1.0);\n"
+"}\n";
+
+static const char* fragment_shader_text =
+"#version 110\n"
+"void main()\n"
+"{\n"
+"    gl_FragColor = vec4(1.0);\n"
+"}\n";
 
 static int swap_tear;
 static int swap_interval;
@@ -43,9 +70,9 @@ static double frame_rate;
 
 static void usage(void)
 {
-    printf("Usage: tearing [-h] [-f]\n");
+    printf("Usage: tearing [-f] [-h]\n");
     printf("Options:\n");
-    printf("  -f create full screen window\n");
+    printf("  -f use full screen\n");
     printf("  -h show this help\n");
 }
 
@@ -53,10 +80,10 @@ static void update_window_title(GLFWwindow* window)
 {
     char title[256];
 
-    sprintf(title, "Tearing detector (interval %i%s, %0.1f Hz)",
-            swap_interval,
-            (swap_tear && swap_interval < 0) ? " (swap tear)" : "",
-            frame_rate);
+    snprintf(title, sizeof(title), "Tearing detector (interval %i%s, %0.1f Hz)",
+             swap_interval,
+             (swap_tear && swap_interval < 0) ? " (swap tear)" : "",
+             frame_rate);
 
     glfwSetWindowTitle(window, title);
 }
@@ -73,11 +100,6 @@ static void error_callback(int error, const char* description)
     fprintf(stderr, "Error: %s\n", description);
 }
 
-static void framebuffer_size_callback(GLFWwindow* window, int width, int height)
-{
-    glViewport(0, 0, width, height);
-}
-
 static void key_callback(GLFWwindow* window, int key, int scancode, int action, int mods)
 {
     if (action != GLFW_PRESS)
@@ -110,60 +132,50 @@ static void key_callback(GLFWwindow* window, int key, int scancode, int action,
         case GLFW_KEY_ESCAPE:
             glfwSetWindowShouldClose(window, 1);
             break;
+
+        case GLFW_KEY_F11:
+        case GLFW_KEY_ENTER:
+        {
+            static int x, y, width, height;
+
+            if (mods != GLFW_MOD_ALT)
+                return;
+
+            if (glfwGetWindowMonitor(window))
+                glfwSetWindowMonitor(window, NULL, x, y, width, height, 0);
+            else
+            {
+                GLFWmonitor* monitor = glfwGetPrimaryMonitor();
+                const GLFWvidmode* mode = glfwGetVideoMode(monitor);
+                glfwGetWindowPos(window, &x, &y);
+                glfwGetWindowSize(window, &width, &height);
+                glfwSetWindowMonitor(window, monitor,
+                                     0, 0, mode->width, mode->height,
+                                     mode->refreshRate);
+            }
+
+            break;
+        }
     }
 }
 
 int main(int argc, char** argv)
 {
-    int ch, width, height;
-    float position;
     unsigned long frame_count = 0;
     double last_time, current_time;
-    int fullscreen = GLFW_FALSE;
-    GLFWmonitor* monitor = NULL;
     GLFWwindow* window;
-
-    while ((ch = getopt(argc, argv, "fh")) != -1)
-    {
-        switch (ch)
-        {
-            case 'h':
-                usage();
-                exit(EXIT_SUCCESS);
-
-            case 'f':
-                fullscreen = GLFW_TRUE;
-                break;
-        }
-    }
+    GLuint vertex_buffer, vertex_shader, fragment_shader, program;
+    GLint mvp_location, vpos_location;
 
     glfwSetErrorCallback(error_callback);
 
     if (!glfwInit())
         exit(EXIT_FAILURE);
 
-    if (fullscreen)
-    {
-        const GLFWvidmode* mode;
+    glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 2);
+    glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 0);
 
-        monitor = glfwGetPrimaryMonitor();
-        mode = glfwGetVideoMode(monitor);
-
-        glfwWindowHint(GLFW_RED_BITS, mode->redBits);
-        glfwWindowHint(GLFW_GREEN_BITS, mode->greenBits);
-        glfwWindowHint(GLFW_BLUE_BITS, mode->blueBits);
-        glfwWindowHint(GLFW_REFRESH_RATE, mode->refreshRate);
-
-        width = mode->width;
-        height = mode->height;
-    }
-    else
-    {
-        width = 640;
-        height = 480;
-    }
-
-    window = glfwCreateWindow(width, height, "", monitor, NULL);
+    window = glfwCreateWindow(640, 480, "Tearing detector", NULL, NULL);
     if (!window)
     {
         glfwTerminate();
@@ -171,7 +183,7 @@ int main(int argc, char** argv)
     }
 
     glfwMakeContextCurrent(window);
-    gladLoadGLLoader((GLADloadproc) glfwGetProcAddress);
+    gladLoadGL(glfwGetProcAddress);
     set_swap_interval(window, 0);
 
     last_time = glfwGetTime();
@@ -179,19 +191,50 @@ int main(int argc, char** argv)
     swap_tear = (glfwExtensionSupported("WGL_EXT_swap_control_tear") ||
                  glfwExtensionSupported("GLX_EXT_swap_control_tear"));
 
-    glfwSetFramebufferSizeCallback(window, framebuffer_size_callback);
     glfwSetKeyCallback(window, key_callback);
 
-    glMatrixMode(GL_PROJECTION);
-    glOrtho(-1.f, 1.f, -1.f, 1.f, 1.f, -1.f);
-    glMatrixMode(GL_MODELVIEW);
+    glGenBuffers(1, &vertex_buffer);
+    glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer);
+    glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);
+
+    vertex_shader = glCreateShader(GL_VERTEX_SHADER);
+    glShaderSource(vertex_shader, 1, &vertex_shader_text, NULL);
+    glCompileShader(vertex_shader);
+
+    fragment_shader = glCreateShader(GL_FRAGMENT_SHADER);
+    glShaderSource(fragment_shader, 1, &fragment_shader_text, NULL);
+    glCompileShader(fragment_shader);
+
+    program = glCreateProgram();
+    glAttachShader(program, vertex_shader);
+    glAttachShader(program, fragment_shader);
+    glLinkProgram(program);
+
+    mvp_location = glGetUniformLocation(program, "MVP");
+    vpos_location = glGetAttribLocation(program, "vPos");
+
+    glEnableVertexAttribArray(vpos_location);
+    glVertexAttribPointer(vpos_location, 2, GL_FLOAT, GL_FALSE,
+                          sizeof(vertices[0]), (void*) 0);
 
     while (!glfwWindowShouldClose(window))
     {
+        int width, height;
+        mat4x4 m, p, mvp;
+        float position = cosf((float) glfwGetTime() * 4.f) * 0.75f;
+
+        glfwGetFramebufferSize(window, &width, &height);
+
+        glViewport(0, 0, width, height);
         glClear(GL_COLOR_BUFFER_BIT);
 
-        position = cosf((float) glfwGetTime() * 4.f) * 0.75f;
-        glRectf(position - 0.25f, -1.f, position + 0.25f, 1.f);
+        mat4x4_ortho(p, -1.f, 1.f, -1.f, 1.f, 0.f, 1.f);
+        mat4x4_translate(m, position, 0.f, 0.f);
+        mat4x4_mul(mvp, p, m);
+
+        glUseProgram(program);
+        glUniformMatrix4fv(mvp_location, 1, GL_FALSE, (const GLfloat*) mvp);
+        glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
 
         glfwSwapBuffers(window);
         glfwPollEvents();
